#  Copyright 2022 Diagnostic Image Analysis Group, Radboudumc, Nijmegen, The Netherlands
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import json
import os
from pathlib import Path
from typing import List

from picai_prep.data_utils import PathLike
from tqdm import tqdm


def generate_mha2nnunet_settings(
    archive_dir: PathLike,
    output_path: PathLike,
    subject_list: List[str],
    task: str = "Task2201_picai_baseline",
    strict: bool = True,
):
    """
    Create mha2nnunet_settings.json for an MHA archive with the following structure:
    /path/to/archive/
    ├── [patient UID]/
        ├── [patient UID]_[study UID]_[modality].mha
        ...

    For each study, the T2-weighted scan (_t2w.mha), apparent diffusion scan (_adc.mha)
    and high b-value scan (_hbv.mha) are collected.

    Parameters:
    - archive_dir: path to MHA archive
    - output_path: path to store MHA->nnUNet settings JSON (parent folder should exist)
    """
    # convert paths
    archive_dir = Path(archive_dir)
    output_path = Path(output_path)

    archive_list = []
    for subject_id in tqdm(subject_list):
        # split subject id
        patient_id, study_id = str(subject_id).split("_")

        # construct path to MRI study
        study_dir = archive_dir / patient_id

        if not study_dir.exists():
            if strict:
                raise FileNotFoundError(f"Folder not found for {subject_id} at {study_dir}!")
            print(f"Folder not found for {subject_id} at {study_dir}! Skipping...")
            continue

        # construct scan paths
        scan_paths = [
            f"{patient_id}/{subject_id}_{modality}.mha"
            for modality in ["t2w", "adc", "hbv"]
        ]
        all_scans_found = all([
            os.path.exists(os.path.join(archive_dir, path))
            for path in scan_paths
        ])

        if all_scans_found:
            # store info for complete studies
            archive_list += [{
                "patient_id": patient_id,
                "study_id": study_id,
                "scan_paths": scan_paths,
            }]
        else:
            if strict:
                raise FileNotFoundError(f"Not all scans found for {subject_id} at {study_dir}!")
            print(f"Not all scans found for {subject_id} at {study_dir}! Skipping...")
            continue

    mha2nnunet_settings = {
        "dataset_json": {
            "task": task,
            "description": "bpMRI scans from PI-CAI dataset to train nnUNet baseline",
            "tensorImageSize": "4D",
            "reference": "",
            "licence": "",
            "release": "1.0",
            "modality": {
                "0": "T2W",
                "1": "CT",
                "2": "HBV"
            },
            "labels": {
                "0": "background",
                "1": "lesion"
            }
        },
        "preprocessing": {
            # optionally, resample and perform centre crop:
            # "matrix_size": [
            #     20,
            #     160,
            #     160
            # ],
            # "spacing": [
            #     3.6,
            #     0.5,
            #     0.5
            # ],
        },
        "archive": archive_list
    }

    if not len(archive_list):
        raise ValueError(f"Did not find any MHA scans in {archive_dir}, aborting.")

    with open(output_path, "w") as fp:
        json.dump(mha2nnunet_settings, fp, indent=4)

    print(f""""
    Saved mha2nnunet_settings to {output_path}, with {len(archive_list)} cases.
    """)



#  Copyright 2022 Diagnostic Image Analysis Group, Radboudumc, Nijmegen, The Netherlands
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
import re
from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List, Union

import SimpleITK as sitk

from picai_prep.converter import Case
from picai_prep.data_utils import PathLike, atomic_image_write
from picai_prep.dcm2mha import (Dicom2MHACase, Dicom2MHAConverter,
                                DICOMImageReader)
from picai_prep.errors import DCESeriesNotFoundError


@dataclass
class Dicom2DCECase(Dicom2MHACase):

    def convert_item(self, output_dir):
        self.initialize()
        self.extract_metadata()
        self._convert_dce(output_dir)

    def _convert_dce(
        self,
        output_dir: PathLike,
        DCE_prefixes: List[str] = None,
        return_image: bool = False,
    ):
        if DCE_prefixes is None:
            DCE_prefixes = [
                # the tags below look like DCE tags, but this is unconfirmed!
                'Perfusie_t1_twist_tra_TTC',
                'Perfusie_t1_twist_tra_TT',
                'Twist_dynamic_Wip576_TT',
                'Perfusie_t1_twist_tra_4mm_TTC',
                'Twist_dynamic_Wip576_pros_TT',
                'Perfusie_t1_twist_tra_3,3mm_TTC',
                'Perfusie_t1_twist_tra_3.3_TTC',
            ]
        if not isinstance(DCE_prefixes, list):
            raise ValueError("DCE_prefixes must be a list")

        # paths
        patient_dir = Path(output_dir) / self.patient_id
        patient_dir.mkdir(parents=True, exist_ok=True)

        # check if file already exists (for joined MHA, i.e., 4D)
        if not return_image:
            dst_path = patient_dir / f"{self.subject_id}_dce.mha"
            if dst_path.exists():
                self.write_log(f"{dst_path} already exists, skipping.")
                return

        # placeholders
        dce_scan_time_map: Dict[str, Path] = {}
        dce_scans: List[sitk.Image] = []

        # collect scan time for all of DCE's T1 scans
        for serie in self.valid_series:
            # Get scan time
            timepoint = None
            for prefix in DCE_prefixes:
                match = re.match(fr'{prefix}?=(?P<time>.+)s', serie.metadata['seriesdescription'])
                if match is not None:
                    timepoint = match.group('time')
                    dce_scan_time_map[timepoint] = serie.path
                    if self.settings.verbose >= 2:
                        self.write_log(f"Got {timepoint} from {serie.metadata['seriesdescription']}")
                    break

            if timepoint is None:
                for prefix in DCE_prefixes:
                    if prefix in serie.metadata['seriesdescription']:
                        # try to get scan time from Acquisition Time
                        timepoint = serie.metadata['acquisitiontime']
                        dce_scan_time_map[timepoint] = serie.path
                        if self.settings.verbose >= 2:
                            self.write_log(f"Got {timepoint} from {serie.metadata['seriesdescription']} ({serie.path})")
                        break

        # Sort scan-time dictionary
        times = dce_scan_time_map.keys()
        times = sorted(times, key=float)

        if self.settings.verbose >= 2:
            self.write_log(f'Sorted times: {times}')

        if len(times) <= 1:
            raise DCESeriesNotFoundError(self.subject_id)

        # Collect all DCE scans in chronological order
        for i, timepoint in enumerate(times):
            ser_dir = dce_scan_time_map[timepoint]
            if self.settings.verbose >= 2:
                self.write_log(f"[{i+1}/{len(times)}]: Reading scan at {timepoint}s from {ser_dir}")

            # Collect T1 image of ordered time points
            image = DICOMImageReader(ser_dir).image
            dce_scans.append(image)

        joined_images: sitk.Image = sitk.JoinSeries(dce_scans)

        # Copy over metadata to joined image
        img = dce_scans[0]
        for key in img.GetMetaDataKeys():
            joined_images.SetMetaData(key, img.GetMetaData(key))

        # Add metadata for the time of the scans
        scan_times = ",".join(times)
        joined_images.SetMetaData("DCE_SCAN_TIMES", scan_times)

        if return_image:
            return joined_images

        # construct target filename and save to file
        atomic_image_write(joined_images, dst_path)


class Dicom2DCEConverter(Dicom2MHAConverter):
    def __init__(
        self,
        input_dir: PathLike,
        output_dir: PathLike,
        dcm2dce_settings: Union[PathLike, Dict] = None,
        case_class: Case = Dicom2DCECase,
    ):
        """
        Convert DCE scans from a DICOM Archive to a single 4D MHA scan.
        Experimental.
        """
        super().__init__(
            input_dir=input_dir,
            output_dir=output_dir,
            dcm2mha_settings=dcm2dce_settings,
            case_class=case_class
        )

    def convert(self):
        self._convert(
            title='Dicom2DCE',
            cases=self.cases,
            parameters={
                'output_dir': self.output_dir
            },
            num_threads=self.settings.num_threads,
        )



#  Copyright 2022 Diagnostic Image Analysis Group, Radboudumc, Nijmegen, The Netherlands
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import os
import shutil

import SimpleITK as sitk
from numpy.testing import assert_allclose
from picai_prep.dcm2dce import Dicom2DCEConverter


def test_dce_conversion(
    input_dir: str = "tests/input/dcm/ProstateX",
    output_dir: str = "tests/output/mha/ProstateX",
    output_expected_dir: str = "tests/output-expected/mha/ProstateX",
):
    """
    Convert DCE series to single 4D MHA.
    """
    # remove output folder (to prevent skipping the conversion)
    if os.path.exists(output_dir):
        shutil.rmtree(output_dir)

    # test usage from command line
    archive = Dicom2DCEConverter(
        input_dir=input_dir,
        output_dir=output_dir,
        dcm2dce_settings="tests/output/dcm2mha_settings.json"
    )
    case = archive.cases[-1]
    case.initialize()
    case.extract_metadata()
    case._convert_dce(output_dir)

    # compare output
    for patient_id, subject_id in [
        ("ProstateX-0218", "ProstateX-0218_02-18-2011"),
    ]:
        for modality in ["dce"]:
            # construct paths to MHA images
            path_out = os.path.join(output_dir, patient_id, f"{subject_id}_{modality}.mha")
            path_out_expected = os.path.join(output_expected_dir, patient_id, f"{subject_id}_{modality}.mha")

            # sanity check: check if outputs exist
            assert os.path.exists(path_out), f"Could not find output file at {path_out}!"
            assert os.path.exists(path_out_expected), f"Could not find output file at {path_out_expected}!"

            # read images
            img = sitk.GetArrayFromImage(sitk.ReadImage(str(path_out)))
            img_expected = sitk.GetArrayFromImage(sitk.ReadImage(str(path_out_expected)))

            # compare images
            assert_allclose(img_expected, img)



#  Copyright 2022 Diagnostic Image Analysis Group, Radboudumc, Nijmegen, The Netherlands
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


from typing import Tuple

import SimpleITK as sitk

from picai_prep.resources.dcm2mha_schema import dcm2mha_schema
from picai_prep.resources.dicom_tags import dicom_tags
from picai_prep.resources.mha2nnunet_schema import mha2nnunet_schema


def plural(num: int, word: str):
    return f"{num} {word}{'' if num == 1 else 's'}"


def make_sitk_readers() -> Tuple[sitk.ImageFileReader, sitk.ImageSeriesReader]:
    """Initialise SimpleITK series and file readers"""
    series_reader = sitk.ImageSeriesReader()
    file_reader = sitk.ImageFileReader()

    for reader in (series_reader, file_reader):
        reader.LoadPrivateTagsOn()

    return file_reader, series_reader


__all__ = [
    # Explicitly expose these functions for easier imports
    "dcm2mha_schema",
    "dicom_tags",
    "mha2nnunet_schema",
]



#  Copyright 2022 Diagnostic Image Analysis Group, Radboudumc, Nijmegen, The Netherlands
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import json
import os
import shutil
from pathlib import Path
from typing import Optional

import SimpleITK as sitk
from numpy.testing import assert_allclose
from picai_prep.data_utils import PathLike
from picai_prep.nnunet2nndet import nnunet2nndet


def test_nnunet2nndet(
    nnunet_raw_data_path: PathLike = "tests/output-expected/nnUNet_raw_data/Task100_test",
    nndet_raw_data_path: PathLike = "tests/output/nnDet_raw_data/Task100_test",
    in_dir_annot: Optional[PathLike] = "labelsTr",
    out_dir_annot: Optional[PathLike] = "raw_splitted/labelsTr",
    in_dir_scans: Optional[PathLike] = "imagesTr",
    out_dir_scans: Optional[PathLike] = "raw_splitted/imagesTr",
    nndet_raw_data_path_expected: PathLike = "tests/output-expected/nnDet_raw_data/Task100_test",
):
    # convert input paths to Path
    nnunet_raw_data_path = Path(nnunet_raw_data_path)
    nndet_raw_data_path = Path(nndet_raw_data_path)
    nndet_raw_data_path_expected = Path(nndet_raw_data_path_expected)

    # remove output folder (to prevent skipping the conversion)
    if os.path.exists(nndet_raw_data_path):
        shutil.rmtree(nndet_raw_data_path)

    # perform conversion from nnUNet to nnDet raw data
    nnunet2nndet(
        nnunet_raw_data_path=nnunet_raw_data_path,
        nndet_raw_data_path=nndet_raw_data_path,
        in_dir_annot=in_dir_annot,
        out_dir_annot=out_dir_annot,
        in_dir_scans=in_dir_scans,
        out_dir_scans=out_dir_scans,
    )

    # check dataset.json
    path_out = nndet_raw_data_path / "dataset.json"
    path_out_expected = nndet_raw_data_path_expected / "dataset.json"
    with open(path_out) as fp1, open(path_out_expected) as fp2:
        assert json.load(fp1) == json.load(fp2)

    # compare output
    for subject_id in [
        "ProstateX-0000_07-07-2011",
        "ProstateX-0001_07-08-2011",
    ]:
        # compare annotations
        for extension in ["nii.gz", "json"]:
            # construct paths
            path_out = nndet_raw_data_path / out_dir_annot / f"{subject_id}.{extension}"
            path_out_expected = nndet_raw_data_path_expected / out_dir_annot / f"{subject_id}.{extension}"

            # sanity check: check if outputs exist
            assert os.path.exists(path_out), f"Could not find output file at {path_out}!"
            assert os.path.exists(path_out_expected), f"Could not find expected output file at {path_out_expected}!"

            if extension == "nii.gz":
                # read images
                img = sitk.GetArrayFromImage(sitk.ReadImage(str(path_out)))
                img_expected = sitk.GetArrayFromImage(sitk.ReadImage(str(path_out_expected)))

                # compare images
                assert_allclose(img_expected, img)
            elif extension == "json":
                with open(path_out) as fp1, open(path_out_expected) as fp2:
                    json_out = json.load(fp1)
                    json_expected = json.load(fp2)
                    assert json_out == json_expected
            else:
                raise ValueError(f"Unexpected extension: {extension}")

        # compare scans
        for modality in ["0000", "0001", "0002"]:
            # construct paths to MHA images
            path_out = nndet_raw_data_path / out_dir_scans / f"{subject_id}_{modality}.nii.gz"
            path_out_expected = nnunet_raw_data_path / in_dir_scans / f"{subject_id}_{modality}.nii.gz"

            # sanity check: check if outputs exist
            assert os.path.exists(path_out), f"Could not find output file at {path_out}!"
            assert os.path.exists(path_out_expected), f"Could not find expected output file at {path_out_expected}!"

            # read images
            img = sitk.GetArrayFromImage(sitk.ReadImage(str(path_out)))
            img_expected = sitk.GetArrayFromImage(sitk.ReadImage(str(path_out_expected)))

            # compare images
            assert_allclose(img_expected, img)



#  Copyright 2022 Diagnostic Image Analysis Group, Radboudumc, Nijmegen, The Netherlands
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


import json
import os
import traceback
from pathlib import Path
from typing import Dict

import numpy as np
import SimpleITK as sitk
from scipy import ndimage
from tqdm import tqdm

from picai_prep.data_utils import (PathLike, atomic_file_copy,
                                   atomic_image_write)


def convert_and_save_label(
    lbl_path_src: PathLike,
    lbl_path_dst: PathLike,
    json_path_dst: PathLike,
):
    if os.path.exists(lbl_path_dst) and os.path.exists(json_path_dst):
        # conversion already done
        print(f"Conversion already done for {lbl_path_src}. Skipping..")
        return

    # placeholder for label info
    label_format: Dict[str, Dict[str, int]] = {
        'instances': {
            # example: str(i+1): 0 for i in range(num_gt_lesions)
        }
    }

    # read label in nnUNet format
    lbl = sitk.ReadImage(str(lbl_path_src))
    lbl_arr = sitk.GetArrayFromImage(lbl)

    # split ground truth lesion/structure annotations as individual 'instances'
    total_num_gt_lesions = 0
    new_labelled_gt = np.zeros_like(lbl_arr)
    for label_value in sorted(np.unique(lbl_arr)):
        # iterate over each lesion/structure type (which all have the same label value for nnUNet)
        if label_value == 0:
            continue

        # split non-overlapping structures
        labelled_gt, num_gt_lesions = ndimage.label(lbl_arr == label_value, np.ones((3, 3, 3)))

        # off-set instance indices with with previously extracted structures
        mask = (labelled_gt > 0)
        labelled_gt[mask] += total_num_gt_lesions

        # add instances to new annotation
        new_labelled_gt[mask] = labelled_gt[mask]

        # add 'metadata' of the extracted instances
        label_format['instances'].update({
            str(i+1): int(label_value - 1)
            for i in range(total_num_gt_lesions, total_num_gt_lesions + num_gt_lesions)
        })

        # count total number of instances
        total_num_gt_lesions += num_gt_lesions

    # convert to SimpleITK
    new_labelled_gt = sitk.GetImageFromArray(new_labelled_gt)
    new_labelled_gt.CopyInformation(lbl)

    # save label in nnDetection format
    atomic_image_write(new_labelled_gt, path=lbl_path_dst)

    # save json description
    with open(json_path_dst, "w") as fp:
        json.dump(label_format, fp)


def nnunet2nndet_dataset_json_conversion(
    path_src: PathLike,
    path_dst: PathLike,
):
    # read dataset.json from nnUNet
    with open(path_src) as fp:
        ds = json.load(fp)

    # ensure dataset config has 'modalities' tag (whereas nnUNet used 'modality')
    if 'modalities' not in ds:
        ds['modalities'] = ds['modality']

    # ensure dataset config has 'dim' tag
    if 'dim' not in ds:
        ds['dim'] = 3

    # change label counting, from 0=background, to 0=first structure of interest
    if "background" in ds["labels"].values():
        # lower label value of each structure type
        labels = [name for name in ds["labels"].values() if name != "background"]
        ds["labels"] = {num: name for num, name in enumerate(labels)}

    # if the target class is not specified, default to 0 (the first structure of interest)
    if "target_class" not in ds:
        ds["target_class"] = 0

    # if no test images are specified, set test_labels to False
    if "test" not in ds or len(ds["test"]) == 0:
        ds["test_labels"] = False

    # save dataset.json in nnDetection format
    with open(path_dst, "w") as fp:
        json.dump(ds, fp, indent=4)


def nnunet2nndet(
    nnunet_raw_data_path: PathLike,
    nndet_raw_data_path: PathLike,
    in_dir_annot: PathLike = "labelsTr",
    out_dir_annot: PathLike = "raw_splitted/labelsTr",
    in_dir_scans: PathLike = "imagesTr",
    out_dir_scans: PathLike = "raw_splitted/imagesTr",
):
    """
    Convert nnU-Net Raw Data Archive to nnDetection Raw Data Archive.
    Each instance needs be isolated for this conversion to be correct.
    """
    # input verification
    nnunet_raw_data_path = Path(nnunet_raw_data_path)
    nndet_raw_data_path = Path(nndet_raw_data_path)
    assert nnunet_raw_data_path.exists(), f"nnUNet data folder not found at {nnunet_raw_data_path}!"

    # make target folders
    nndet_raw_data_path.mkdir(parents=True, exist_ok=True)
    if out_dir_annot is not None:
        out_dir_annot = nndet_raw_data_path / out_dir_annot
        out_dir_annot.mkdir(parents=True, exist_ok=True)
        print(f"Copying annotations from {nnunet_raw_data_path / in_dir_annot} to {out_dir_annot}")
    if out_dir_scans is not None:
        in_dir_scans = nnunet_raw_data_path / in_dir_scans
        out_dir_scans = nndet_raw_data_path / out_dir_scans
        out_dir_scans.mkdir(parents=True, exist_ok=True)
        print(f"Copying scans from {in_dir_scans} to {out_dir_scans}")

    # copy dataset.json
    path_src = nnunet_raw_data_path / "dataset.json"
    path_dst = nndet_raw_data_path / "dataset.json"
    if os.path.exists(path_dst):
        print("dataset.json already copied. Skipping..")
    else:
        nnunet2nndet_dataset_json_conversion(
            path_src=path_src,
            path_dst=path_dst,
        )

    # convert annotations from nnUNet format to nnDetection format
    if out_dir_annot is not None:
        # collect list of annotation files in nnUNet data folder
        files = os.listdir(nnunet_raw_data_path / in_dir_annot)
        files = [fn for fn in files if ".nii.gz" in fn and "._" not in fn]

        # collect list of case names
        subject_list = [fn.replace(".nii.gz", "") for fn in files]
        subject_list = sorted(subject_list)
        print(f"Found {len(subject_list)} annotations")

        for subject_id in tqdm(subject_list):
            # construct paths
            lbl_path_src = nnunet_raw_data_path / in_dir_annot / f"{subject_id}.nii.gz"
            lbl_path_dst = out_dir_annot / f"{subject_id}.nii.gz"
            json_path_dst = out_dir_annot / f"{subject_id}.json"

            # convert nnUNet label to nnDetection format
            try:
                convert_and_save_label(
                    lbl_path_src=lbl_path_src,
                    lbl_path_dst=lbl_path_dst,
                    json_path_dst=json_path_dst,
                )
            except RuntimeError:
                print(f"Error for {lbl_path_src}:")
                print(traceback.format_exc())

    # copy scans from nnUNet data folder to nnDetection data folder
    if out_dir_scans is not None:
        for fn in os.listdir(in_dir_scans):
            # construct paths
            src_path = in_dir_scans / fn
            dst_path = out_dir_scans / fn

            if os.path.exists(dst_path):
                # conversion already done
                print(f"Conversion already done for {src_path}. Skipping..")
                continue

            # copy file
            atomic_file_copy(
                src_path=src_path,
                dst_path=dst_path,
            )



import setuptools

if __name__ == '__main__':
    with open("README.md", "r", encoding="utf-8") as fh:
        long_description = fh.read()

    setuptools.setup(
        version='2.1.11',
        author_email='Joeran.Bosma@radboudumc.nl',
        long_description=long_description,
        long_description_content_type="text/markdown",
        url='https://github.com/DIAGNijmegen/picai_prep',
        project_urls={
            "Bug Tracker": "https://github.com/DIAGNijmegen/picai_prep/issues"
        },
        license='Apache License, Version 2.0',
        package_dir={"": "src"},  # our packages live under src, but src is not a package itself
        packages=setuptools.find_packages('src', exclude=['tests']),
        exclude_package_data={'': ['tests']},
    )
